//
//  GameKitSynchronizationServer.m
//  TicTacToe
//
//  Created by Ingo Kasprzak on 11.10.12.
//  Copyright (c) 2012 Silutions. All rights reserved.
//

#define GAMEKIT_EXECUTION_DELAY 0.35f

// Keys for synchronization dictionary
#define SyncDatabaseSyncCommandKey @"command"
#define SyncDatabaseSyncDataKey @"data"
#define SyncDatabaseSyncDateKey @"date"
#define SyncDatabaseSyncSenderUIDKey @"senderUID"
#define SyncDatabaseSyncDataUIDKey @"dataUID"
#define SyncDatabaseSyncWantsReceiptKey @"wantsReceipt"


#import "GameKitSynchronizationServer.h"
#import "SynchronizationServer.h"
#import "Device+Methods.h"

@interface GameKitSynchronizationServer ()
@property (strong) GKSession *gkSession;
@end

@implementation GameKitSynchronizationServer

#pragma mark - Initialization

- (BOOL)startServer {
    if (!self.gkSession) {
		[super startServer];

        // Standard GameKit initialization
        GKSessionMode sessionMode = GKSessionModePeer;
		
		self.gkSession = [[GKSession alloc] initWithSessionID:nil
												  displayName:nil
												  sessionMode:sessionMode];
		self.gkSession.delegate = self;
		[self.gkSession setDataReceiveHandler:self withContext:nil];
		self.gkSession.available = YES;
        
		DBLog(@"GameKit initialized.");
	}

	return YES;
}

- (BOOL)stopServer {
	if (self.gkSession) {
		DBLog(@"Stopping GameKit");
		[super stopServer];

		[self.gkSession disconnectFromAllPeers];
		self.gkSession.available = NO;
		[self.gkSession setDataReceiveHandler:nil withContext:nil];
		self.gkSession.delegate = nil;
		self.gkSession = nil;
	}
	
    return YES;
}

- (BOOL)sendData:(NSData*)data toPeers:peerIDs {
	dispatch_after(dispatch_time(DISPATCH_TIME_NOW, GAMEKIT_EXECUTION_DELAY * NSEC_PER_SEC),
				   dispatch_get_main_queue(),
				   ^(void){
					   [self.gkSession sendData:data
										toPeers:peerIDs
								   withDataMode:GKSendDataReliable
										  error:nil];
				   });
	
    return YES;
}

- (BOOL)sendData:(NSData*)data toDevices:(NSArray*)deviceUIDs {
	NSArray *peerIDs = [self peerIdentifiersForDeviceUIDs:deviceUIDs];
	return [self sendData:data toPeers:peerIDs];
}

#pragma mark - Game Kit

- (void)session:(GKSession *)session connectionWithPeerFailed:(NSString *)peerID withError:(NSError *)error {
	DBLog(@"%@", error);
}

- (void)session:(GKSession *)session didFailWithError:(NSError *)error {
	DBLog(@"%@", error);
}

- (void)session:(GKSession *)session didReceiveConnectionRequestFromPeer:(NSString *)peerID {
		[session acceptConnectionFromPeer:peerID error:nil];
		DBLog(@"Accepted connection from peer %@.", peerID);
}

- (void)session:(GKSession *)session peer:(NSString *)peerID didChangeState:(GKPeerConnectionState)state {
	switch (state) {
		case GKPeerStateAvailable: {
			DBLog(@"Peer %@ is available", peerID);
            // Time Delay. Sonst klappt der Connect unter verschiedenen iOS-Versionen nicht immer
			dispatch_after(dispatch_time(DISPATCH_TIME_NOW, GAMEKIT_EXECUTION_DELAY * NSEC_PER_SEC),
						   dispatch_get_main_queue(),
						   ^(void){
							   [session connectToPeer:peerID withTimeout:5.0f];
						   });
			DBLog(@"Trying to connect to %@", peerID);
		}
			break;
		case GKPeerStateConnected: {
			[self sendObject:self.delegate.thisDevice.uid
				 withCommand:SyncDatabaseSyncIdentificationCommand
					 toPeers:@[peerID]
				wantsReceipt:NO];
			DBLog(@"Successfully connected to peer %@", peerID);
		}
			break;
		case GKPeerStateUnavailable:
			DBLog(@"Peer %@ is unavailable", peerID);
			break;
		case GKPeerStateDisconnected: {
			NSString* deviceUID = [self deviceUIDForPeerID:peerID];
			[self removePeerIdentifier:peerID];
			[self receivedDisconnection:deviceUID fromPeer:peerID];
			DBLog(@"Disconnected from peer %@", peerID);
		}
			break;
		case GKPeerStateConnecting:
			DBLog(@"Connecting to peer %@", peerID);
			break;
		default:
			DBLog(@"Peer %@ has unknown state", peerID);
			break;
	}
}

- (void)receiveData:(NSData *)data fromPeer:(NSString *)peer inSession: (GKSession *)session context:(void *)context {
	[self receivedData:data fromPeer:peer];
}

#pragma mark - Overridden methods

- (void)receivedIdentification:(NSString *)deviceUID fromPeer:(NSString *)peer {
	[self addConnectedDevice:deviceUID withPeerIdentifier:peer];
	[super receivedIdentification:deviceUID fromPeer:peer];
}

@end
